@@ -29,7 +29,7 @@ from util import graphics, ALIGN_LEFT, ALIGN_RIGHT, ALIGN_CENTER, ALIGN_FILL, \
     PAPER_SIZES
 from util.lib import debug
 from widgetPresenter import BoxEditor, PagesToExportDialog, FileDialog, \
-    PageSizeDialog, getPopupMenu, QuestionDialog, UnpaperDialog
+    PageSizeDialog, getPopupMenu, WarningDialog, UnpaperDialog
 import gettext
 import gobject
 import gtk
@@ -37,6 +37,7 @@ import math
 import os.path
 import pygtk
 import threading
+import sys
 pygtk.require('2.0')
 _ = gettext.gettext
 
@@ -125,6 +126,7 @@ class SourceImagesSelectorIconView(gtk.IconView):
         self.set_columns(1)
         self.set_reorderable(True)
         self.add_events(gtk.gdk.BUTTON_PRESS_MASK)
+        self.set_selection_mode(gtk.SELECTION_BROWSE)
         self.connect('button-press-event', self.pressedRightButton)
     
     def pressedRightButton(self, target, event):
@@ -156,7 +158,8 @@ class SourceImagesSelectorIconView(gtk.IconView):
     
 class ImageReviewer:
     
-    def __init__(self, path_to_image, ocr_engines):
+    def __init__(self, main_window, path_to_image, ocr_engines):
+        self.main_window = main_window
         self.path_to_image = path_to_image
         self.text_box_fill_color = (94, 156, 235, 150)
         self.text_box_stroke_color = (94, 156, 235, 250)
@@ -167,6 +170,8 @@ class ImageReviewer:
         self.selectable_boxes_area.connect('removed_box', self.removedBox)
         self.selectable_boxes_area.connect('updated_box', self.updatedBox)
         self.selectable_boxes_area.connect('dragged_box', self.updatedBoxBounds)
+        self.selectable_boxes_area.connect('deselected_box',
+                                           self.deselectedBoxCb)
         self.image_pixbuf = gtk.gdk.pixbuf_new_from_file(self.path_to_image)
         self.reviewer_area = gtk.HPaned()
         self.reviewer_area.set_position(500)
@@ -218,6 +223,10 @@ class ImageReviewer:
             num_boxes = self.boxeditor_notebook.get_n_pages()
             self.addBoxEditor(box)
             self.boxeditor_notebook.set_current_page(num_boxes)
+        self.updateMainWindow()
+
+    def deselectedBoxCb(self, widget, box):
+        self.updateMainWindow()
     
     def updatedBox(self, widget, box):
         for editor in self.editor_list:
@@ -230,6 +239,7 @@ class ImageReviewer:
                 editor.updateBounds(box)
     
     def removedBox(self, widget, box):
+        self.updateMainWindow()
         for i in xrange(len(self.editor_list)):
             editor = self.editor_list[i]
             if editor.box == box:
@@ -240,10 +250,16 @@ class ImageReviewer:
         return False
     
     def __getPageNumFromBox(self, box):
+        editor = self.__getEditorFromBox(box)
+        if editor:
+            return self.boxeditor_notebook.page_num(editor.box_editor)
+        return -1
+
+    def __getEditorFromBox(self, box):
         for editor in self.editor_list:
             if editor.box == box:
-                return self.boxeditor_notebook.page_num(editor.box_editor)
-        return -1
+                return editor
+        return None
     
     def applyTextColors(self):
         self.selectable_boxes_area.fill_color_rgba = self.text_box_fill_color
@@ -260,12 +276,23 @@ class ImageReviewer:
             self.addBoxEditor(box)
     
     def performOcrForAllEditors(self, engine = None):
-        for editor in self.editor_list:
+        self.performOcrForEditors(self.editor_list, engine)
+
+    def performOcrForSelectedBoxes(self, engine = None):
+        selected_boxes = self.selectable_boxes_area.getSelectedAreas()
+        self.performOcrForEditors([self.__getEditorFromBox(box) \
+                                   for box in selected_boxes],
+                                  engine)
+
+    def performOcrForEditors(self, editors_list, engine = None):
+        for editor in editors_list:
+            if editor == None:
+                continue
             editor.performOcr(engine)
             editor.performClassification(engine)
             if editor.box_editor.getType() == IMAGE_TYPE:
                 editor.box_editor.setText('')
-    
+
     def __getAllDataBoxes(self):
         boxes = []
         for editor in self.editor_list:
@@ -321,22 +348,29 @@ class ImageReviewer:
             changed = True
         if changed:
             self.selectable_boxes_area.zoom(min(image_height, image_width), False)
+
+    def updateMainWindow(self):
+        has_selected_areas = self.selectable_boxes_area.getSelectedAreas()
+        self.main_window.setHasSelectedBoxes(bool(has_selected_areas))
     
 class ImageReviewer_Controler:
     
-    def __init__(self, notebook, images_dict, source_images_selector_widget, ocr_engines, configuration_manager, tripple_statusbar, selection_changed_signal = 'selection-changed'):
-        self.notebook = notebook
+    def __init__(self, main_window, images_dict, source_images_selector_widget,
+                 ocr_engines, configuration_manager,
+                 selection_changed_signal = 'selection-changed'):
+        self.main_window = main_window
+        self.notebook = self.main_window.notebook
         self.image_reviewer_dict = {}
         self.source_images_selector_widget = source_images_selector_widget
         self.ocr_engines = ocr_engines
         self.configuration_manager = configuration_manager
-        self.tripple_statusbar = tripple_statusbar
+        self.tripple_statusbar = self.main_window.tripple_statusbar
         for key, image in images_dict.items():
             self.addImage(key, image)
         self.source_images_selector_widget.connect(selection_changed_signal, self.selectImageReviewer)
     
     def addImage(self, pixbuf, image):
-        image_reviewer = ImageReviewer(image, self.ocr_engines)
+        image_reviewer = ImageReviewer(self.main_window, image, self.ocr_engines)
         image_reviewer.selectable_boxes_area.connect('changed_zoom', self.__setZoomStatus)
         image_reviewer.setTextFillColor(self.configuration_manager.getTextFill())
         image_reviewer.setTextStrokeColor(self.configuration_manager.getTextStroke())
@@ -364,7 +398,11 @@ class ImageReviewer_Controler:
     
     def __setZoomStatus(self, widget, zoom):
         self.tripple_statusbar.left_statusbar.insert(_('Zoom') + ': ' + str(int(zoom * 100)) + '%')
-    
+
+    def recognizeSelectedAreas(self, widget):
+        image_reviewer = self.__getCurrentReviewer()
+        image_reviewer.performOcrForSelectedBoxes(self.configuration_manager.favorite_engine)
+
     def performBoxDetection(self, widget):
         image_reviewer = self.__getCurrentReviewer()
         self.performBoxDetectionForReviewer(image_reviewer)
@@ -381,7 +419,13 @@ class ImageReviewer_Controler:
             window_size = None
         else:
             window_size = float(window_size)
-        image_processor = ImageProcessor(image_reviewer.path_to_image, window_size)
+        try:
+            image_processor = ImageProcessor(image_reviewer.path_to_image, window_size)
+        except:
+            warning_dialog = WarningDialog(str(sys.exc_info()[1]))
+            warning_dialog.run()
+            warning_dialog.destroy()
+            return
         while gtk.events_pending():
             gtk.main_iteration()
         block_retriever = BlockRetriever(image_processor.imageToBinary())
@@ -575,11 +619,11 @@ class ImageReviewer_Controler:
             reviewer.setImageStrokeColor(self.configuration_manager.getImageStroke())
             reviewer.updateBoxesColors()
     
-    def zoomIn(self, zoom_value = 0.2):
+    def zoomIn(self, zoom_value = 0.05):
         current_reviewer = self.__getCurrentReviewer()
         current_reviewer.selectable_boxes_area.zoom(zoom_value)
     
-    def zoomOut(self, zoom_value = -0.2):
+    def zoomOut(self, zoom_value = -0.05):
         current_reviewer = self.__getCurrentReviewer()
         current_reviewer.selectable_boxes_area.zoom(-abs(zoom_value))
 
