@@ -73,6 +73,7 @@ class MainWindow:
         </menu>
         <menu action="Document">
             <menuitem action="OCRFeederDetection"/>
+            <menuitem action="RecognizeAreas"/>
         </menu>
         <menu action="Tools">
             <menuitem action="OCREngines"/>
@@ -135,12 +136,12 @@ class MainWindow:
                                   ('Quit', gtk.STOCK_QUIT, _('_Quit'), None, _('Exit the program'), menu_items['exit']),
                                   ('OpenProject', gtk.STOCK_OPEN, _('_Open'), None, _('Open project'), menu_items['open_project']),
                                   ('SaveProject', gtk.STOCK_SAVE, _('_Save'), None, _('Save project'), menu_items['save_project']),
-                                  ('SaveProjectAs', gtk.STOCK_SAVE_AS, _('_Save As') + '...', '<control><shift>s', _('Save project with a chosen name'), menu_items['save_project_as']),
+                                  ('SaveProjectAs', gtk.STOCK_SAVE_AS, _('_Save As…'), '<control><shift>s', _('Save project with a chosen name'), menu_items['save_project_as']),
                                   ('AddImage', gtk.STOCK_ADD, _('_Add Image'), None, _('Add another image'), menu_items['add_image']),
                                   ('AddFolder', gtk.STOCK_ADD, _('Add _Folder'), None, _('Add all images in a folder'), menu_items['add_folder']),
                                   ('AppendProject', gtk.STOCK_ADD, _('Append Project'), None, _('Load a project and append it to the current one'), menu_items['append_project']),
                                   ('ImportPDF', gtk.STOCK_ADD, _('_Import PDF'), None, _('Import PDF'), menu_items['import_pdf']),
-                                  ('Export', None, _('_Export...'), '<control><shift>e', _('Export to a chosen format'), menu_items['export_dialog']),
+                                  ('Export', None, _('_Export…'), '<control><shift>e', _('Export to a chosen format'), menu_items['export_dialog']),
                                   ('Edit', None, _('_Edit')),
                                   ('EditPage', gtk.STOCK_EDIT, _('_Edit Page'), None, _('Edit page settings'), menu_items['edit_page']),
                                   ('Preferences', gtk.STOCK_PREFERENCES, _('_Preferences'), None, _('Configure the application'), menu_items['preferences']),
@@ -158,6 +159,7 @@ class MainWindow:
                                   ('Help', None, _('_Help')),
                                   ('About', gtk.STOCK_ABOUT, _('_About'), None, _('About this application'), menu_items['about']),
                                   ('OCRFeederDetection', None, _('_Recognize Document'), None, _("Automatic Detection and Recognition"), tool_items['detection']),
+                                  ('RecognizeAreas', None, _('Recognize Selected _Areas'), None, _("Recognize Selected Areas"), menu_items['recognize_areas']),
                                   ('GenerateODT', None, _('_Generate ODT'), None, _("Export to ODT"), tool_items['export_to_odt']),
                                   ])
         ui_manager.insert_action_group(action_group, 0)
@@ -196,6 +198,14 @@ class MainWindow:
                    'AppendProject', 'ZoomFit']
         for gtkaction in [self.action_group.get_action(action) for action in actions]:
             gtkaction.set_sensitive(has_images)
+
+    def setHasSelectedBoxes(self, has_selected_boxes = True):
+        if not self.action_group:
+            return
+        actions = ['RecognizeAreas']
+        for gtkaction in [self.action_group.get_action(action) \
+                          for action in actions]:
+            gtkaction.set_sensitive(has_selected_boxes)
         
 class BoxEditor(gtk.ScrolledWindow):
     
@@ -433,7 +443,7 @@ class BoxEditor(gtk.ScrolledWindow):
         self.angle_spin = gtk.SpinButton(gtk.Adjustment(0.0, -360.0, 360.0, 0.1, 100.0, 0.0), 1.0, 1)
         self.detect_angle_button = gtk.Button(_('Detect'))
         hbox = gtk.HBox()
-        hbox.add(gtk.Label(_('Angle') + ':'))
+        hbox.add(gtk.Label(_('Angle:')))
         hbox.add(self.angle_spin)
         vbox = gtk.VBox()
         vbox.pack_start(hbox, False)
@@ -597,7 +607,7 @@ class PageSizeDialog(gtk.Dialog):
         self.paper_sizes = gtk.combo_box_new_text()
         papers = PAPER_SIZES.keys()
         papers.sort()
-        self.paper_sizes.append_text(_('Custom') + '...')
+        self.paper_sizes.append_text(_('Custom…'))
         for paper in papers:
             self.paper_sizes.append_text(paper)
         active_index = self.__checkIfSizeIsStandard(page_size)
@@ -789,7 +799,7 @@ class UnpaperDialog(gtk.Dialog):
         unpapered_image = os.path.splitext(name)[0] + '_unpapered.ppm'
         unpapered_image = lib.getNonExistingFileName(unpapered_image)
         command += ' %s %s' % (name, unpapered_image)
-        progress_bar = CommandProgressBarDialog(command, _('Performing Unpaper'), _('Performing unpaper. Please wait...'))
+        progress_bar = CommandProgressBarDialog(command, _('Performing Unpaper'), _('Performing unpaper. Please wait…'))
         progress_bar.run()
         self.unpapered_image = unpapered_image
     
@@ -859,7 +869,7 @@ class CommandProgressBarDialog(gtk.Dialog):
         try:
             self.process = subprocess.Popen(self.command.split(), stdout = subprocess.PIPE, stderr = subprocess.STDOUT, bufsize=1)
         except:
-            warning = SimpleDialog(_('An error occured!'), _('Error'), 'warning')
+            warning = SimpleDialog(_('An error occurred!'), _('Error'), 'warning')
             warning.run()
             return False
         self.timer = gobject.timeout_add(100, self.__pulse)
@@ -870,7 +880,7 @@ class CommandProgressBarDialog(gtk.Dialog):
         exit_value = self.process.poll()
         if exit_value != None:
             if exit_value != 0:
-                warning = SimpleDialog(_('An error occured!'), _('Error'), 'warning')
+                warning = SimpleDialog(_('An error occurred!'), _('Error'), 'warning')
                 warning.run()
             self.destroy()
             return False
@@ -955,7 +965,7 @@ class PreferencesDialog(gtk.Dialog):
         self.notebook.append_page(general_box, gtk.Label(_('Appearance')))
     
     def __makeColors(self):
-        colors_frame = PlainFrame(_('Select boxes colors'))
+        colors_frame = PlainFrame(_('Select boxes\' colors'))
         self.text_fill_color = self.__getColorButton(self.configuration_manager.text_fill)
         self.text_stroke_color = self.__getColorButton(self.configuration_manager.text_stroke)
         self.image_fill_color = self.__getColorButton(self.configuration_manager.image_fill)
@@ -1054,7 +1064,45 @@ class PreferencesDialog(gtk.Dialog):
         color_button.set_use_alpha(True)
         color_button.set_alpha(values[3] << 8)
         return color_button
-        
+
+class SystemEnginesDialog(gtk.Dialog):
+
+    def __init__(self, engines):
+        super(SystemEnginesDialog, self).__init__(_('OCR Engines'),
+                                                  flags = gtk.DIALOG_MODAL |
+                                                          gtk.DIALOG_DESTROY_WITH_PARENT,
+                                                  buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                                                             gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT))
+        self.set_size_request(300, -1)
+        self.set_icon_from_file(WINDOW_ICON)
+        self.list_store = gtk.ListStore(bool, str, gobject.TYPE_PYOBJECT)
+        for engine in engines:
+            self.list_store.append((True, engine.name, engine))
+        self.vbox.pack_start(self.__makeMainArea(), True, True, 0)
+        self.vbox.show_all()
+
+    def __makeMainArea(self):
+        frame = PlainFrame(_('Engines to be added'))
+        self.tree_view = gtk.TreeView(self.list_store)
+        renderer = gtk.CellRendererToggle()
+        renderer.set_property('activatable', True)
+        renderer.connect('toggled', self.__includeEngineToggledCb, 0)
+        column = gtk.TreeViewColumn(_('Include'), renderer)
+        column.add_attribute(renderer, "active", 0)
+        self.tree_view.append_column(column)
+        renderer = gtk.CellRendererText()
+        column = gtk.TreeViewColumn(_('Engine'), renderer, text = 1)
+        self.tree_view.append_column(column)
+        self.tree_view.set_search_column(1)
+        frame.add(self.tree_view)
+        return frame
+
+    def __includeEngineToggledCb(self, renderer, path, column):
+        self.list_store[path][column] = not self.list_store[path][column]
+
+    def getChosenEngines(self):
+        return [engine for include, name, engine in self.list_store if include]
+
 class OcrManagerDialog(gtk.Dialog):
     
     def __init__(self, engines_manager):
@@ -1073,6 +1121,7 @@ class OcrManagerDialog(gtk.Dialog):
         self.delete_engine.connect('clicked', self.__delete)
         self.new_engine.connect('clicked', self.__engine_settings)
         self.edit_engine.connect('clicked', self.__edit)
+        self.detect_engines.connect('clicked', self.__detectEnginesCb)
     
     def __makeMainArea(self):
         frame = PlainFrame(_('OCR Engines'))
@@ -1086,10 +1135,12 @@ class OcrManagerDialog(gtk.Dialog):
         self.new_engine = gtk.Button(_('Add'), gtk.STOCK_ADD)
         self.delete_engine = gtk.Button(_('Delete'), gtk.STOCK_DELETE)
         self.edit_engine = gtk.Button(_('Edit'), gtk.STOCK_EDIT)
+        self.detect_engines = gtk.Button(_('Detect'))
         buttons_box = gtk.VBox()
         buttons_box.pack_start(self.new_engine, False)
         buttons_box.pack_start(self.edit_engine, False)
         buttons_box.pack_start(self.delete_engine, False)
+        buttons_box.pack_start(self.detect_engines, False, False, 10)
         engines_box.pack_end(buttons_box, False)
         frame.add(engines_box)
         return frame
@@ -1141,6 +1192,24 @@ class OcrManagerDialog(gtk.Dialog):
         for engine in self.engines_manager.getEnginesNames():
             self.list_store.append((engine,))
 
+    def __detectEnginesCb(self, button):
+        engines = self.engines_manager.configuration_manager.getEnginesInSystem()
+        if not engines:
+            info = InfoDialog(_('No OCR engines were found in the system.\n'
+                                'Please make sure you have OCR engines installed '
+                                'and available.'),
+                              _('No OCR engines available'))
+            info.run()
+            info.destroy()
+            return
+        engines_dialog = SystemEnginesDialog(engines)
+        response = engines_dialog.run()
+        if response == gtk.RESPONSE_ACCEPT:
+            for engine in engines_dialog.getChosenEngines():
+                self.engines_manager.addNewEngine(engine)
+        engines_dialog.destroy()
+        self.__getEngines()
+
 class OcrSettingsDialog(gtk.Dialog):
     
     def __init__(self, engine_manager, engine = None):
@@ -1177,7 +1246,7 @@ class OcrSettingsDialog(gtk.Dialog):
         self.engine_path_entry = gtk.Entry()
         self.__packSettingInFrame(table, 3, _('Engine Path'), self.engine_path_entry, engine_path, _('The path to the engine program'))
         self.arguments_entry = gtk.Entry()
-        self.__packSettingInFrame(table, 4, _('Engine arguments'), self.arguments_entry, arguments, _('Arguments, use $IMAGE for image and $FILE if it writes to a file'))
+        self.__packSettingInFrame(table, 4, _('Engine arguments'), self.arguments_entry, arguments, _('Arguments: use $IMAGE for image and $FILE if it writes to a file'))
         return table
     
     def setEngine(self):
@@ -1193,7 +1262,7 @@ class OcrSettingsDialog(gtk.Dialog):
                 self.engine_manager.addNewEngine(engine)
             return True
         except:
-            SimpleDialog(_('Error setting the new engine, please check your engine settings.'), _('Warning'), 'warning').run()
+            SimpleDialog(_('Error setting the new engine; please check your engine settings.'), _('Warning'), 'warning').run()
             print sys.exc_info()
             return False
     
@@ -1224,10 +1293,7 @@ class CustomAboutDialog(gtk.AboutDialog):
         self.set_license(GPL_STATEMENT)
         self.set_artists(OCRFEEDER_STUDIO_ARTISTS)
         self.set_comments(_(OCRFEEDER_STUDIO_COMMENTS))
-        translator_credits = ''
-        for translator in OCRFEEDER_STUDIO_TRANSLATORS:
-            translator_credits += translator[0] + ': ' + translator[1] + '\n'
-        self.set_translator_credits(translator_credits)
+        self.set_translator_credits(_('translator-credits'))
         self.set_icon_from_file(WINDOW_ICON)
 
 def getPopupMenu(menus_info):
